@@ -1,6 +1,3 @@
-# Copyright 2012 OpenStack Foundation
-# All Rights Reserved.
-#
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
 # a copy of the License at
@@ -13,9 +10,14 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from heatclient.common import template_utils
+from __future__ import print_function
 
+import os
+import sys
+
+from heatclient.common import template_utils
 from oslo_serialization import jsonutils
+from oslo_utils import encodeutils
 from oslo_utils import importutils
 import prettytable
 import six
@@ -23,22 +25,41 @@ import yaml
 
 from senlinclient.common import exc
 from senlinclient.common.i18n import _
-from senlinclient.openstack.common import cliutils
 
 
-# Using common methods from oslo cliutils
-# Will change when the module graduates
-arg = cliutils.arg
-env = cliutils.env
-print_list = cliutils.print_list
-exit = cliutils.exit
-
 supported_formats = {
     "json": lambda x: jsonutils.dumps(x, indent=2),
     "yaml": lambda x: yaml.safe_dump(x, default_flow_style=False)
 }
 
 
+def arg(*args, **kwargs):
+    """Decorator for CLI args."""
+
+    def _decorator(func):
+        if not hasattr(func, 'arguments'):
+            func.arguments = []
+
+        if (args, kwargs) not in func.arguments:
+            func.arguments.insert(0, (args, kwargs))
+
+        return func
+
+    return _decorator
+
+
+def env(*args, **kwargs):
+    """Returns the first environment variable set.
+
+    If all are empty, defaults to '' or keyword arg `default`.
+    """
+    for arg in args:
+        value = os.environ.get(arg)
+        if value:
+            return value
+    return kwargs.get('default', '')
+
+
 def import_versioned_module(version, submodule=None):
     module = 'senlinclient.v%s' % version
     if submodule:
@@ -54,13 +75,14 @@ def format_nested_dict(d, fields, column_names):
     for n in column_names:
         pt.align[n] = 'l'
 
-    for field in d.keys():
+    keys = sorted(d.keys())
+    for field in keys:
         value = d[field]
-        if value is six.string_types:
-            pt.add_row([field, value])
-        else:
-            pt.add_row([field, jsonutils.dumps(value, indent=2,
-                                               ensure_ascii=False)])
+        if not isinstance(value, six.string_types):
+            value = jsonutils.dumps(value, indent=2, ensure_ascii=False)
+        if value is None:
+            value = '-'
+        pt.add_row([field, value.strip('"')])
 
     return pt.get_string()
 
@@ -77,6 +99,56 @@ def list_formatter(record):
     return '\n'.join(record or [])
 
 
+def _print_list(objs, fields, formatters=None, sortby_index=0,
+                mixed_case_fields=None, field_labels=None):
+    """Print a list of objects as a table, one row per object.
+
+    :param objs: iterable of :class:`Resource`
+    :param fields: attributes that correspond to columns, in order
+    :param formatters: `dict` of callables for field formatting
+    :param sortby_index: index of the field for sorting table rows
+    :param mixed_case_fields: fields corresponding to object attributes that
+        have mixed case names (e.g., 'serverId')
+    :param field_labels: Labels to use in the heading of the table, default to
+        fields.
+    """
+    formatters = formatters or {}
+    mixed_case_fields = mixed_case_fields or []
+    field_labels = field_labels or fields
+    if len(field_labels) != len(fields):
+        raise ValueError(_("Field labels list %(labels)s has different number "
+                           "of elements than fields list %(fields)s"),
+                         {'labels': field_labels, 'fields': fields})
+
+    if sortby_index is None:
+        kwargs = {}
+    else:
+        kwargs = {'sortby': field_labels[sortby_index]}
+    pt = prettytable.PrettyTable(field_labels)
+    pt.align = 'l'
+
+    for o in objs:
+        row = []
+        for field in fields:
+            if field in formatters:
+                data = formatters[field](o)
+            else:
+                if field in mixed_case_fields:
+                    field_name = field.replace(' ', '_')
+                else:
+                    field_name = field.lower().replace(' ', '_')
+                data = getattr(o, field_name, '')
+            if data is None:
+                data = '-'
+            row.append(data)
+        pt.add_row(row)
+
+    if six.PY3:
+        print(encodeutils.safe_encode(pt.get_string(**kwargs)).decode())
+    else:
+        print(encodeutils.safe_encode(pt.get_string(**kwargs)))
+
+
 def print_list(objs, fields, formatters=None, sortby_index=0,
                mixed_case_fields=None, field_labels=None):
     # This wrapper is needed because sdk may yield a generator that will
@@ -85,10 +157,10 @@ def print_list(objs, fields, formatters=None, sortby_index=0,
         objs = []
 
     try:
-        cliutils.print_list(objs, fields, formatters=formatters,
-                            sortby_index=sortby_index,
-                            mixed_case_fields=mixed_case_fields,
-                            field_labels=field_labels)
+        _print_list(objs, fields, formatters=formatters,
+                    sortby_index=sortby_index,
+                    mixed_case_fields=mixed_case_fields,
+                    field_labels=field_labels)
     except Exception as ex:
         exc.parse_exception(ex)
 
@@ -101,15 +173,31 @@ def print_dict(d, formatters=None):
 
     for field in d.keys():
         if field in formatters:
-            pt.add_row([field, formatters[field](d[field])])
+            data = formatters[field](d[field])
         else:
-            pt.add_row([field, d[field]])
-    print(pt.get_string(sortby='Property'))
+            data = d[field]
+        if data is None:
+            data = '-'
+        pt.add_row([field, data])
+
+    content = pt.get_string(sortby='Property')
+    if six.PY3:
+        print(encodeutils.safe_encode(content).decode())
+    else:
+        print(encodeutils.safe_encode(content))
+
+
+def print_action_result(rid, res):
+    if res[0] == "OK":
+        output = _("accepted by action %s") % res[1]
+    else:
+        output = _("failed due to '%s'") % res[1]
+    print(_(" %(cid)s: %(output)s") % {"cid": rid, "output": output})
 
 
 def format_parameters(params, parse_semicolon=True):
-    '''Reformat parameters into dict of format expected by the API.'''
-    if not params:
+    """Reformat parameters into dict of format expected by the API."""
+    if not params or params == ['{}']:
         return {}
 
     if parse_semicolon:
@@ -167,7 +255,7 @@ def process_stack_spec(spec):
     new_spec = {
         # TODO(Qiming): add context support
         'disable_rollback': spec.get('disable_rollback', True),
-        'context':  spec.get('context', {}),
+        'context': spec.get('context', {}),
         'parameters': spec.get('parameters', {}),
         'timeout': spec.get('timeout', 60),
         'template': template,
@@ -185,3 +273,9 @@ def format_output(output, format='yaml'):
     except KeyError:
         raise exc.HTTPUnsupported(_('The format(%s) is unsupported.')
                                   % fmt)
+
+
+def exit(msg=''):
+    if msg:
+        print(msg, file=sys.stderr)
+    sys.exit(1)
